'use strict'
var assert = require('assert')
var equal = assert.deepEqual
var gitSemverTags = require('./')
var mocha = require('mocha')
var it = mocha.it
var shell = require('shelljs')
var writeFileSync = require('fs').writeFileSync
var gitDummyCommit = require('git-dummy-commit')

shell.config.silent = true
shell.rm('-rf', 'tmp')
shell.mkdir('tmp')
shell.cd('tmp')
shell.exec('git init')

it('should error if no commits found', function (done) {
  gitSemverTags(function (err) {
    assert(err)

    done()
  })
})

it('should get no semver tags', function (done) {
  writeFileSync('test1', '')
  shell.exec('git add --all && git commit -m"First commit"')
  shell.exec('git tag foo')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, [])

    done()
  })
})

it('should get the semver tag', function (done) {
  writeFileSync('test2', '')
  shell.exec('git add --all && git commit -m"Second commit"')
  shell.exec('git tag v2.0.0')
  writeFileSync('test3', '')
  shell.exec('git add --all && git commit -m"Third commit"')
  shell.exec('git tag va.b.c')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['v2.0.0'])

    done()
  })
})

it('should get both semver tags', function (done) {
  shell.exec('git tag v3.0.0')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['v3.0.0', 'v2.0.0'])

    done()
  })
})

it('should get all semver tags if two tags on the same commit', function (done) {
  shell.exec('git tag v4.0.0')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['v4.0.0', 'v3.0.0', 'v2.0.0'])

    done()
  })
})

it('should still work if I run it again', function (done) {
  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['v4.0.0', 'v3.0.0', 'v2.0.0'])

    done()
  })
})

it('should be in reverse chronological order', function (done) {
  writeFileSync('test4', '')
  shell.exec('git add --all && git commit -m"Fourth commit"')
  shell.exec('git tag v1.0.0')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['v1.0.0', 'v4.0.0', 'v3.0.0', 'v2.0.0'])

    done()
  })
})

it('should work with prerelease', function (done) {
  writeFileSync('test5', '')
  shell.exec('git add --all && git commit -m"Fifth commit"')
  shell.exec('git tag 5.0.0-pre')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['5.0.0-pre', 'v1.0.0', 'v4.0.0', 'v3.0.0', 'v2.0.0'])

    done()
  })
})

it('should work with empty commit', function (done) {
  shell.rm('-rf', '.git')
  shell.exec('git init')
  gitDummyCommit('empty commit')
  shell.exec('git tag v1.1.0')
  shell.exec('git tag blarg-project@1.0.0') // should be ignored.
  gitDummyCommit('empty commit2')
  gitDummyCommit('empty commit3')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['v1.1.0'])

    done()
  })
})

it('should work with lerna style tags', function (done) {
  writeFileSync('test5', '')
  shell.exec('git add --all && git commit -m"sixth commit"')
  shell.exec('git tag foo-project@4.0.0')
  shell.exec('git add --all && git commit -m"seventh commit"')
  shell.exec('git tag foo-project@5.0.0')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['foo-project@5.0.0', 'foo-project@4.0.0', 'blarg-project@1.0.0'])
    done()
  }, {lernaTags: true})
})

it('should work with lerna style tags with multiple digits', function (done) {
  writeFileSync('test5', '')
  shell.exec('git add --all && git commit -m"fifth commit"')
  shell.exec('git tag foobar-project@0.0.10')
  shell.exec('git add --all && git commit -m"sixth commit"')
  shell.exec('git tag foobar-project@0.10.0')
  shell.exec('git add --all && git commit -m"seventh commit"')
  shell.exec('git tag foobar-project@10.0.0')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['foobar-project@10.0.0', 'foobar-project@0.10.0', 'foobar-project@0.0.10', 'foo-project@5.0.0', 'foo-project@4.0.0', 'blarg-project@1.0.0'])
    done()
  }, {lernaTags: true})
})

it('should allow lerna style tags to be filtered by package', function (done) {
  writeFileSync('test5', '')
  shell.exec('git add --all && git commit -m"seventh commit"')
  shell.exec('git tag bar-project@5.0.0')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['bar-project@5.0.0'])
    done()
  }, {lernaTags: true, package: 'bar-project'})
})

it('should not allow package filter without lernaTags=true', function (done) {
  gitSemverTags(function (err) {
    equal(err.message, 'opts.package should only be used when running in lerna mode')
    done()
  }, {package: 'bar-project'})
})

it('should work with tag prefix option', function (done) {
  writeFileSync('test6', '')
  shell.exec('git add --all && git commit -m"eigth commit"')
  shell.exec('git tag ms/6.0.0')
  shell.exec('git add --all && git commit -m"tenth commit"')
  shell.exec('git tag ms/7.0.0')
  shell.exec('git add --all && git commit -m"eleventh commit"')
  shell.exec('git tag notms/7.0.0')

  gitSemverTags(function (err, tags) {
    if (err) done(err)
    equal(tags, ['ms/7.0.0', 'ms/6.0.0'])
    done()
  }, {tagPrefix: 'ms/'})
})
